/*
   This sketch is used to drive an bike-chain as a clock, 12 chain segments with 18 chainlinks each. The stepper motor drives a 12 tooth gear
   resulting in 1.5 turns or 300 full steps per hour (NEMA 17 with 200 steps/turn) I used a quarter step setting which results in a 3 second 
   per step advance. Since the Stepper motor is mounted on a board, it was very noisy with the A4988 driver. I switched to TMC2130 (non-SPI mode) in 1/4 step stealth mode 
   which reduced noise drastically besides some high pitch tone when the stepper was idle. The TMC2130 are A4988 drop-in replacements as long they are in non-SPI mode (SPI jumper closed)
   This whining sound resulted from the chain slightly pushing back on the motor while the driver was keeping the position. This was eliminated 
   after driving the chain two steps forward and going back one step. This reduced the pushback from the chain after the step was complete.
   The rotary switch is used to rotate through the color-wheel for the display while idle and button press will be used to step through time/date settings
   
   The weekday display is a roller driven by a 28BYJ-48 motor

   Written by: Patrick Geschwindner
   13-Oct-2022: Code created

  For Display intensity adjustment use either a pot connected to A6 or LDR connected between GND and Pin A6 with a resistor of 10k from A6 to VCC
  The map statement for reading the LDR values at the beginning of the "show-LEDs" routine may need adjustment based on the LDR type

Wire RTC and OLED display:
  VCC +5v
  GND GND
  SDA -> Analog pin 4
  SCL -> Analog pin 5

A4988 Stepper
  Step -> D5 
  Direction -> D6 
  MS2 -> Logic VCC for quarter step
  
TMC2130 Stepper (non-SPI mode)
  Step -> D5 
  Direction -> D6 (A4988 and TMC2130 have opposite direction sense, change direction using variable chain_dir and WKday_dir to 1 or -1)
  CFG1 pin -> Logic VCC for 1/4 Step Stealth mode
  
 28BYJ-48 Stepper
 
*/

// libary declarations:
#include <AccelStepper.h>  //include the libary we will use for the stepper motor
#include <Wire.h>          //include the libary that handles i2c communication (SDA/SCL pin A4/A5)
#include "RTClib.h"        //include the libary that we use for the real time clock

#include <TimeLib.h>
#include "FastLED.h"
#define ENCODER_OPTIMIZE_INTERRUPTS  // rotary encoder using pin 2,3 for Arduino Nano
#include <Encoder.h>
#include "OneButton.h"  // menu button pin 4

#include <SSD1306Ascii.h>  // text only light library
#include "SSD1306AsciiWire.h"

// 0X3C+SA0 - 0x3C or 0x3D
#define I2C_ADDRESS 0x3C  // OLED display address

SSD1306AsciiWire lcd;
bool disp_sleep = false;
bool prev_disp_sleep = false;
unsigned long dispTimer = 30;  // dislay timeout in seconds

// The next two definition lines set the time and weekday position at which the chain and weekday roller stop moving after hall effect (or reed contact) 
// sensor detection/calibration. These are just initial values for my clock and depends where Reed contacts and magnets are positioned.
// Remember the time displayed by the chain after it reached the calibration point. he chin will stop and wait for the weekday roller to reach its calibration point.
// Set the hr/min/sec below (Year, Month, Day, Hour, minute, second) for variable 'CalibrationTime'. For my clock it was 11:29
// Year, Month and Day are not important, we only need the 12h offset for the day (hence modulo 43200)

long CalibrationTime = (DateTime(2020, 8, 20, 11, 29, 0).unixtime()) % 43200; // adjust this to match time display after finding home position
long WKday_offset = 100;            // change this to shift Weekday display showing the right day after all moves done. One day is a 7th of a turn = 582 halfsteps

int WKday_pos[7] = {3462,4046,552,1134,1716,2298,2880}; // Weekday roller position depending on magnet/reed position (Monday to Sunday)
//int WKday_pos[7] = { 3492,0 ,582, 1164, 1746, 2328, 2910};    // Other combination I tested for my implementation
//int WKday_pos[7] = { 0, 582, 1164, 1746, 2328, 2910, 3492 };  // and one more combination I tested for my implementation

RTC_DS1307 rtc;         // tell the libary which real time clock module we use
time_t time_provider()  // define regular time-sync with RTC
{
  return rtc.now().unixtime();  // get time/date in unix epoch time
}
char monthString[37] = { "JanFebMarAprMayJunJulAugSepOctNovDec" }; // clear text display for adjusting month

AccelStepper Stepper1(1, 5, 6);          // AccelStepper Mode 1 (Chain NEMA17 Driver A4988/TMC2208/TMC2130 two pins, D5 = Pulse/Step, D6 = Direction)
AccelStepper Stepper2(8, 8, 10, 9, 11);  // AccelStepper Mode 8 Halfstep mode (Weekday 28BYJ-48 Driver four pins, IN1 to IN4, Arduino PIN 8 to 11) 4076 steps/rev in halfstep mode
                                         // Stepper pin sequence defined as IN-1,3,2,4 otherwise there could be issues with reverse operation and torque will be low

//Rotary Switch D2,D3 - Menu Button D4 (Encoder works best when using interrupts)
Encoder Enc(3, 2);  // Reverse pin numbers if encoder counts in the wrong direction
int EncPos = 0;
int EncDir = 0;
byte menu = 0;

OneButton SelBtn(4, true, true);  // set button1 connected to port A3, activeLOW, enable pullup

// LDR connects to ground and A6, A6 via 10k to 5v
#define LDRpin 6  // analog pin A6

// All LEDs are NeoPixel in sequence. Number of leds in your strip
#define NUM_LEDS 73   // see definitions and positions below
#define DATA_PIN 7    // Neopixel / WS2812B data
CRGB leds[NUM_LEDS];  // Define the array of leds
#define FRAMES_PER_SECOND 120

// Led strip position definitions. LED-Group starting at running LED number on strip
#define LEDoffset_WKday 0            // (3 LEDs)
#define LEDoffset_Minutes 3          // (38 LEDs)
#define LEDoffset_Tube_Right 41      // (1 LED) 
#define LEDoffset_Hour_Background 42 // (11 LEDs)
#define LEDoffset_Tube_Left 53       // (1 LED)
#define LEDoffset_Tube_Middle 54     // (1 LED)
#define LEDoffset_Moon 55            // (18 LEDs)

// Variable declaration:
int chain_dir = 1;  // used to switch direction if your clock runs backwards (set to 1/-1 to change direction)
int WKday_dir = -1;
long chain_steps;    // the number of steps the chain needs to move

long clk_position;                                            //clock position in seconds based on 12h (0 - 43199)
DateTime T0;                                                  // time from RTC time
time_t T0sec;                                                 // time in epoch

int prev_sec = 0;      // remember previous second update
int bck_color = 50;    // Starting Hue of background color
int fgd_color = 180;   // Starting Hue of foreground color for minute display (minute indicator)
int corrected_hue = 0; // if different types of LEDs are in use we may need coor compensation
int HueOffset = 10;
byte dispBrightness = 1;

// Definitions
#define TimeHall_pin 12       //pin for chain hall effect sensor (or reed relay) signal, low means found position
#define WKdayHall_pin A0      //pin for WKday hall effect sensor (or reed relay) signal, low means found position
#define Chain_fast_speed 300  //fast speed of the stepper
#define Chain_slow_speed 20   //slow speed of the stepper
#define WKday_fast_speed 400  //fast speed of the stepper (2038 steps per rev using full step / 4076 for half step)
#define WKday_slow_speed 100  //slow speed of the stepper
// clock gear calculation:
// sec/hour = 3600
// Motor Full rotation = 200 steps, 1 hour = 1.5 rotations (18 chain links) = 300 steps per hour.
// NEMA 17 Motor 200 full steps = 12 sec / step
// half step = 6 sec / step
// quarter step = 3 sec / step
#define Sec_per_step 3L         //ammount of seconds per step for chain
#define Overshoot_Steps 1       //overshoot chain position and return back for reducing noise while not moving
unsigned long dispTimeout = 0;  // next display timeout event in millis

//moonphase variables
byte Moon_phase = 0;
int y = 0, m = 0, d = 0, h = 0;


//==============================================================================================================================
void setup() {  // the setup routine is run only once and used to configure certain things

  Serial.begin(9600);  // start serial communication with the computer for debug purposes
  Wire.begin();        // start I2C


  rtc.begin();  // startup real time clock
  lcd.begin(&Adafruit128x64, I2C_ADDRESS);
  lcd.clear();
  lcd.setFont(Cooper19);

  setSyncProvider(time_provider);  //sets Time Library to RTC time
  setSyncInterval(3600);           //sync Time Library to RTC every hour

  //------------------------------------------------------------------------
  // following line sets the RTC to the date & time this sketch was compiled, uncomment and upload this once to set the RTC afterwards comment out again
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  //------------------------------------------------------------------------

  FastLED.setBrightness(dispBrightness);  // setup the FastLED display
  FastLED.addLeds<WS2812, DATA_PIN, GRB>(leds, NUM_LEDS);

  pinMode(TimeHall_pin, INPUT_PULLUP);  // configure the pin connected to the hall effect sensor as input
  pinMode(WKdayHall_pin, INPUT_PULLUP);

  Stepper1.setMaxSpeed(Chain_fast_speed);              //set max speed the stepper motor will turn (steps/second)
  Stepper1.setAcceleration((0.7 * Chain_fast_speed));  //set max acceleration the stepper will use (steps/second^2)
  Stepper2.setMaxSpeed(WKday_fast_speed);
  Stepper2.setAcceleration((0.7 * WKday_fast_speed));

  SelBtn.attachClick(click1);
  dispTimeout = millis() + dispTimer * 1000;  // time for display timeout
  upd_disp();                                 // populate display with time and date
  showLEDwalk();                              // LED test during startup
  getPhase();                                 // calculate moon phase for initial display
  show_leds();                                // show backlight and moon
  calibrate();    // since we just powered up, we dont know the position of chain and weekday, go calibrate
}


//====================================================================================================================================
void loop() {  // the main part of the code that loops over and over

  encoder_check();  // check encoder action
  SelBtn.tick();    // check menu/select Button
  //if (menu == 0) {  // keep both motors moving while steps are pending during idle mode
  //  Stepper1.run();
  //  Stepper2.run();
  //}
  if (second() != prev_sec)  // branch off once a second
  {
    prev_sec = second();  // remember recent second
    calc_steps();         // use this void to calculate if, and how many steps are neccecary, calculate WKday roller as well
    //if (menu == 0) { upd_disp(); }       // update display
    upd_disp();
    if (second() == 0) { getPhase(); }  // calculate moon phase once a minute only
    show_leds();                        // update LED patterns
  }

  //debug();
  //----------------------------------------------------------------------------------------------------
  //write your own added feature code here

  //----------------------------------------------------------------------------------------------------
  //delay(10);  // delay to stabilise the loop
}


//====================================================================================================================================



//====================================================================================================================================
void calibrate() {
  //This void finds the calibration points for the chain and weekday roller.
  //after this we move from the now known position to the actual time/day we want to display


  Stepper1.setSpeed(chain_dir * Chain_fast_speed);  // set the chain stepper to use fast speed
  Serial.println(F("Calibrating-Fast Mode Hour Chain......"));
  while (digitalRead(TimeHall_pin))  // while we have not reached the hall effect sensor, waiting for low
  {
    Stepper1.runSpeed();  // run the engine at constant speed
  }
  Serial.println(F("Position found, moving back"));
  Stepper1.move(-100 * chain_dir);                  // move back a litte bit to compensate for overshoot of fast movement or if it accidentally was already triggered at start
  Stepper1.runToPosition();                         // command the engine to actually move
  Stepper1.setSpeed(chain_dir * Chain_slow_speed);  // set the stepper to use slow speed
  Serial.println(F("Calibrating-Slow Mode......"));
  while (digitalRead(TimeHall_pin))  // while we have not reached the hall effect sensor
  {
    Stepper1.runSpeed();  // run the engine at constant speed
  }

  Stepper2.setSpeed(WKday_dir * WKday_fast_speed);  // set the weekday stepper to use fast speed
  Serial.println(F("Calibrating-Fast Mode Weekday......"));
  while (digitalRead(WKdayHall_pin))  // while we have not reached the hall effect sensor, waiting for low
  {
    Stepper2.runSpeed();  // run the engine at constant speed
  }
  Serial.println(F("Position found, moving back"));
  Stepper2.move(-180 * WKday_dir);                  // move back a litte bit to compensate for overshoot of fast movement or if it accidentally was already triggered at at start
  Stepper2.runToPosition();                         // command the engine to actually move
  Stepper2.setSpeed(WKday_dir * WKday_slow_speed);  // set the stepper to use slow speed
  Serial.println(F("Calibrating-Slow Mode......"));
  while (digitalRead(WKdayHall_pin))  // while we have not reached the hall effect sensor
  {
    Stepper2.runSpeed();  // run the engine at constant speed
  }
  delay(1500);                                      // delay to give the user a chance to read the time so it can be in variable "CalibrationTime" at top
  Stepper1.setSpeed(chain_dir * Chain_fast_speed);  // set chain speed back to fast mode
  Stepper2.setSpeed(WKday_dir * WKday_fast_speed);  // set weekday roller back to fast mode
  Stepper2.setCurrentPosition(WKday_offset);        // use offset for correct display
  clk_position = CalibrationTime;                   // time at calibration pause display in TOD seconds based on 12h (0 - 43199)
  // Next: use current dat/time from RTC for sync time
}


//====================================================================================================================================
void calc_steps() {
  long tim_tod_seconds = 0;  // running time
  //long clk_tod_seconds = 0; // clock position time
  long d_seconds = 0;                                            // seconds between time and current clock display
  tim_tod_seconds = hour() * 3600L + minute() * 60L + second();  // calculate seconds of the day from current time
  d_seconds = tim_tod_seconds - clk_position;                    // difference in seconds

  if (d_seconds < 0) {
    d_seconds = d_seconds + 86400;  // we leaped over midnight, avoiding the clock would rotate two rounds backward to 0
  }
  if (d_seconds > 43199)  // we have an analog clock so it only displays 12 hours, e.g. instead going 16 hours we can reduce this by going 4 hours
  {
    d_seconds = d_seconds - 43200;
  }

  if (d_seconds > 21599)  // if the difference is bigger than +/- 6 hours its faster to turn in the opposite direction
  {
    d_seconds = d_seconds - 43200;
  }
  chain_steps = d_seconds / Sec_per_step;  //convert the seconds to steps

  if (chain_steps != 0)  // move chain stepper and check WKday stepper only if a step is needed
  {
    move_stepper();  // the move stepper void will make the actual movement based on the variable "chain_steps"
  }
  //Serial.println("=== new iteration ===");
  //Serial.print("CLK at: ");
  //Serial.print(clk_position);
  //Serial.print(" - ");
  //Serial.print(hour(clk_position));
  //Serial.print(":");
  //Serial.print(minute(clk_position));
  //Serial.print(":");
  //Serial.println(second(clk_position));
  //Serial.print("Diff: ");
  //Serial.print(d_seconds);
  //Serial.print(" Steps: ");
  //Serial.println(chain_steps);
  //Serial.print("Weekday: ");
  //Serial.println(weekday());
  //Serial.println();
}


//====================================================================================================================================
void move_stepper() {
  if (menu > 0) { return; }                // prevent steppers to run while adjusting date/time
  Stepper1.move(chain_dir * chain_steps + Overshoot_Steps);  //tell the motor how many steps are needed and in which direction plus overshoot to reduce noise while not moving
  Stepper1.runToPosition();                //run the stepper to the new position.

  Stepper1.move(chain_dir * -1 * Overshoot_Steps);  //return back to intended position 
  Stepper1.runToPosition();                //run the stepper to the new position.

  clk_position = clk_position + chain_steps * Sec_per_step;  //remember new position
  if (clk_position > 43199) {
    clk_position = clk_position - 43200;  // 12h display correction
  }
  //Serial.print("Weekday: ");
  //Serial.println(weekday());
  Stepper2.moveTo(WKday_dir * WKday_pos[weekday() - 1]);  //tell the motor which position to go to, library remembers current position
  Stepper2.runToPosition();                               //run the stepper to the new position.

  chain_steps = 0;  // reset steps after new position has been reached
}

//====================================================================================================================================
void show_leds() {
  // NUM_LEDS (73) leds - Numbering:
  // Led strip position definitions (just repeated here for clarity)
  // LEDoffset_WKday            0
  // LEDoffset_Minutes          3   (different sort of LEDs with colorwheel about 20 ahead of others, needs compensation)
  // LEDoffset_Tube_Right      41
  // LEDoffset_Hour_Background 42
  // LEDoffset_Tube_Left       53
  // LEDoffset_Tube_Middle     54
  // LEDoffset_Moon            55
int val = analogRead(LDRpin);                   // LDRpin to analog input
  dispBrightness = map(val, 300, 1023, 200, 30);  // map input to brightness levels. LDR is connected to GND, mapping is inverse

  int z = 0;

  // set Minute display LEDs
  
  // trying to correct hue for denser LED strip (144/m with ifferent color representation)
corrected_hue = bck_color+HueOffset;
if (corrected_hue > 255) {corrected_hue -= 255;}
if (corrected_hue < 0) {corrected_hue += 255;}

  for (z = LEDoffset_Minutes; z < LEDoffset_Minutes + 38; z++) {  // set all leds to background color
    leds[z] = CHSV(corrected_hue, 200, 200);
  }
  z = ((minute() * 60 + second()) % 600);
  z = map(z, 0, 585, 37, 0);                                // minute pointer, 10 minutes over 38 LEDs, reverse direction (37 to 0). Use 585 to show last LED for 15 seconds
  leds[z + LEDoffset_Minutes] = CHSV(fgd_color, 200, 200);  // set led indicating seconds to foreground color

  // set Weekday LEDS to background color
  for (z = LEDoffset_WKday; z < LEDoffset_WKday + 3; z++) {  // set leds to background color
    leds[z] = CHSV(bck_color, 200, 255);
  }

  // illumiate Tubes
z = LEDoffset_Tube_Right;
   leds[z] = CHSV(bck_color, 200, 255);
z = LEDoffset_Tube_Left;
   leds[z] = CHSV(bck_color, 200, 255);
z = LEDoffset_Tube_Middle;
   leds[z] = CHSV(bck_color, 200, 255);

  // illuminate hour background
     for (z = LEDoffset_Hour_Background; z < LEDoffset_Hour_Background + 11; z++) {  // set all leds to background color
    leds[z] = CHSV(bck_color, 200, 255);
  }

  // Moon display
  for (z = LEDoffset_Moon; z < LEDoffset_Moon + 18; z++) {  // set all leds to background color
    leds[z] = CRGB::Black;
  }

  if (Moon_phase < 17) {
    for (z = 0; z < (Moon_phase - 2); z = z + 3) {
      leds[z + LEDoffset_Moon] = CHSV(bck_color, 200, 255); // or use plain White: = CRGB::White;
      leds[z + 1 + LEDoffset_Moon] = CHSV(bck_color, 200, 255);
      leds[z + 2 + LEDoffset_Moon] = CHSV(bck_color, 200, 255);
    }
  }

  if ((Moon_phase >= 17) && (Moon_phase < 39)) {
    for (int z = 0; z < 18; z++) {
      leds[z + LEDoffset_Moon] = CHSV(bck_color, 200, 255);
    }
    for (int z = 0; z < (Moon_phase - 21); z = z + 3) {
      leds[z + LEDoffset_Moon] = CRGB::Black;
      leds[z + 1 + LEDoffset_Moon] = CRGB::Black;
      leds[z + 2 + LEDoffset_Moon] = CRGB::Black;
    }
  }

  FastLED.setBrightness(dispBrightness);
  FastLED.show();
}


//====================================================================================================================================
void click1() {
  //Serial.println(F("Button pressed"));
  dispTimeout = millis() + dispTimer * 1000;  // time for display timeout
  menu++;
  if (menu > 5) { menu = 0; } // menu 0 = idle/color-wheel, 1 = hour, 2 = minute, 3 = day, 4 = month, 5 = year
  EncMove();  // no direction detected yet, but updating display with setting indicator
}

//====================================================================================================================================
void upd_disp() {

  if (millis() > dispTimeout) {  // check if we crossed inactivity timeout, clear screen and mark display a sleep
    disp_sleep = true;
  } else {
    disp_sleep = false;
  }
  if (prev_disp_sleep != disp_sleep) { // avoid clearing display over and over if sleep is active
    if (disp_sleep) { lcd.clear(); }
    prev_disp_sleep = disp_sleep;
  }
  if (disp_sleep) { return; }


  //Serial.println("starting Display refresh");

  String thisTime = "";
  if (hour() < 10) { thisTime += "0"; }  // add leading zero if required
  thisTime += String(hour()) + ":";
  if (minute() < 10) { thisTime += "0"; }  // add leading zero if required
  thisTime += String(minute()) + ":";
  if (second() < 10) { thisTime += "0"; }  // add leading zero if required
  thisTime += String(second());
  const char* newTime = (const char*)thisTime.c_str();

  String thisDate = "";
  if (day() < 10) { thisDate += "0"; }  // add leading zero if required
  thisDate += String(day()) + "-";
  for (int i = 0; i <= 2; i++) {
    thisDate += monthString[((month() - 1) * 3) + i];  // add Month in 3-letter formt
  }
  thisDate += "-";
  if (year() - 2000 < 10) { thisDate += "0"; }  // add leading zero if required
  thisDate += String(year() - 2000);
  const char* newDate = (const char*)thisDate.c_str();

  lcd.setCursor(20, 0);
  lcd.print(newTime);
  lcd.setCursor(10, 3);
  lcd.print(newDate);
  lcd.setCursor(20, 5);
  switch (menu) {
    case 0:
      lcd.print("          ");
      break;
    case 1:  // Change hour
      lcd.print("Hour");
      break;
    case 2:  // Change Minute and reset seconds when leaving to next item
      lcd.print("Minute  ");
      break;
    case 3:  // Change day
      lcd.print("   Day     ");
      break;
    case 4:  // Change month
      lcd.print("Month   ");
      break;
    case 5:  // Change year
      lcd.print("Year    ");
      break;
    default:
      break;
  }

}

//====================================================================================================================================
void encoder_check() {
  EncDir = 0;  // resetEncoder Direction result

  long EncPos = Enc.read(); // one step is +/- 4 counts
  if (EncPos > 3)           // changed forward -> increment once for 4 steps (3 and higher)
  {
    EncDir = 1;
  }
  if (EncPos < -3) {        // reverse
    EncDir = -1;
  }
  if (EncDir != 0) {
    Enc.write(0);           // reset position of encoder
    EncPos = 0;
    EncMove();              // Act on Encoder change
  }
  //Serial.println(newPosition);
  //check_menu();
}

//====================================================================================================================================
void updRTC() {  // unused, was another way updating time from RTC
  //T0 = rtc.now();         // Get time from RTC
  //T0sec = T0.unixtime();  // generate epoch time from RTC
  //setTime(hour(T0sec), minute(T0sec), second(T0sec), month(T0sec), day(T0sec), year(T0sec));  // set HW clock

  //  T0sec = T0sec + 30;
  //rtc.adjust(DateTime(year(T0sec), month(T0sec), day(T0sec), hour(T0sec), minute(T0sec), second(T0sec)));
}

//====================================================================================================================================
void EncMove() {
  int tmp = 0;  // temp value to change Month and Year

  switch (menu) {
    case 0:  /// Change color wheel while in idle menu
      bck_color += EncDir*5;
      if (bck_color > 255) { bck_color = 0; }
      if (bck_color < 0) { bck_color = 255; }
      fgd_color += EncDir*5;
      if (fgd_color > 255) { fgd_color = 0; }
      if (fgd_color < 0) { fgd_color = 255; }

      break;
    case 1:                            // Change hour
      setTime(now() + EncDir * 3600);  // Change Hour
      break;
    case 2:
      setTime(now() + EncDir * 60);  // Change Minute and reset seconds
      setTime(hour(), minute(), 0, day(), month(), year());
      break;
    case 3:
      setTime(now() + EncDir * 86400);  // Change day
      break;
    case 4:
      tmp = month() + EncDir;  // Change month
      if (tmp < 1) { tmp = 12; }
      if (tmp > 12) { tmp = 1; }
      setTime(hour(), minute(), second(), day(), tmp, year());
      break;
    case 5:
      tmp = year() + EncDir;  // Change year
      if (tmp < 2000) { tmp = 2000; }
      if (tmp > 2099) { tmp = 2099; }
      setTime(hour(), minute(), second(), day(), month(), tmp);
      break;
    default:
      // statements
      break;
  }

  if (menu > 0) { rtc.adjust(DateTime(year(), month(), day(), hour(), minute(), second())); }  // Sync RTC after change



  dispTimeout = millis() + dispTimer * 1000;  // reset time for display timeout

  getPhase();  // calculate moon phase 
  upd_disp();  // update OLED
  show_leds(); // update LEDs
}


//====================================================================================================================================
void showLEDwalk() {

  // test all LEDs
  int z = 0;
  int val = analogRead(LDRpin);                   // LDRpin to analog input
  dispBrightness = map(val, 300, 1023, 200, 30);  // map input to brightness levels. LDR is connected to GND, mapping is inverse

  FastLED.setBrightness(dispBrightness);

  for (z = 0; z < NUM_LEDS; z++) {  // set all leds to background color
    leds[z] = CHSV(bck_color, 200, 255);
    FastLED.show();
    delay(30);
  }
  for (z = 0; z < NUM_LEDS; z++) {  // set all leds to background color
    leds[z] = CRGB::Black;
    FastLED.show();
    delay(30);
  }
}

//====================================================================================================================================
void getPhase() {  // calculate the current phase of the moon
  int Y = year();
  int M = month();
  int D = day();
  double AG, IP;                // based on the current date
                                // algorithm adapted from Stephen R. Schmitt's
                                // Lunar Phase Computation program, originally
  long YY, MM, K1, K2, K3, JD;  // written in the Zeno programming language
                                // http://home.att.net/~srschmitt/lunarphasecalc.html
  // calculate julian date
  YY = Y - floor((12 - M) / 10);
  MM = M + 9;
  if (MM >= 12)
    MM = MM - 12;

  K1 = floor(365.25 * (YY + 4712));
  K2 = floor(30.6 * MM + 0.5);
  K3 = floor(floor((YY / 100) + 49) * 0.75) - 38;

  JD = K1 + K2 + D + 59;
  if (JD > 2299160)
    JD = JD - K3;

  IP = normalize((JD - 2451550.1) / 29.530588853);
  AG = IP * 29.53;
  Moon_phase = IP * 39;
  //Serial.print(F("Moon Phase: "));
  //Serial.println(Moon_phase);
}

double normalize(double v) {  // normalize moon calculation between 0-1
  v = v - floor(v);
  if (v < 0)
    v = v + 1;
  return v;
}